import logging

import pathlib
import pytest
from pytest import fixture, raises

from type_hint_checker.exceptions import IncorrectFileException
from type_hint_checker.main import check_type_hints, filter_files


NO_RETURN = "tests/cases/no_return.py"
MIXED_ARGS = "tests/cases/mixed_parameters.py"
NO_ARGS = "tests/cases/no_parameters.py"
NOT_A_FUNCTION = "tests/cases/not_a_function.py"
STRING_TYPE_HINT = "tests/cases/string_type_hint.py"
MIXED_ARGS_WITH_RETURN = "tests/cases/mixed_parameters_with_return.py"
COMMENT_ABOVE = "tests/cases/comment_above.py"
COMMENT_BELOW = "tests/cases/comment_below.py"
COMMENT_BODY = "tests/cases/comment_body.py"
MANY_COMMENTS = "tests/cases/many_comments.py"
DIFFERENT_COMMENT = "tests/cases/different_comment.py"
COMMENT_HEADER = "tests/cases/comment_header.py"
COMMENT_LONG_HEADER = "tests/cases/comment_long_header.py"
COMMENT_LONG_HEADER_2 = "tests/cases/comment_long_header_2.py"
EMPTY_CLASS = "tests/cases/empty_class.py"
MIXED_ARGS_CLASS = "tests/cases/mixed_parameters_class.py"
NO_RETURN_CLASS = "tests/cases/no_return_class.py"
PROPERLY_ANNOTATED_CLASS = "tests/cases/properly_annotated_class.py"
STATIC_FUNCTION_CLASS = "tests/cases/static_function_class.py"
ANNOTATED_SELF_CLASS = "tests/cases/annotated_self_class.py"


@fixture
def incorrect_file() -> str:
    """Example of an incorrect file"""
    return "key1:\nkey2:"


@pytest.mark.parametrize(
    "input_path,result",
    [
        (MIXED_ARGS, False),
        (NO_RETURN, False),
        (NO_ARGS, True),
        (NOT_A_FUNCTION, True),
        (STRING_TYPE_HINT, True),
        (MIXED_ARGS_WITH_RETURN, False),
        (COMMENT_ABOVE, False),
        (COMMENT_BELOW, False),
        (COMMENT_BODY, True),
        (MANY_COMMENTS, True),
        (COMMENT_HEADER, True),
        (COMMENT_LONG_HEADER, True),
        (COMMENT_LONG_HEADER_2, True),
    ],
)
def test_check_type_hints(input_path: str, result: bool) -> None:
    assert check_type_hints([input_path]) == result


@pytest.mark.parametrize(
    "input_path,ignore_comment,result",
    [
        (COMMENT_BODY, "jdncajbc", False),
        (COMMENT_BODY, "custom", False),
        (DIFFERENT_COMMENT, "custom", True),
    ],
)
def test_custom_ignore_comment(
    input_path: str, ignore_comment: str, result: bool
) -> None:
    assert check_type_hints([input_path], ignore_comment=ignore_comment) == result


def test_incorrect_file(incorrect_file, tmp_path: pathlib.Path) -> None:
    """Test if passing incorrect file raises the correct error"""
    file = tmp_path / "file737ny73814781.py"
    file.write_text(incorrect_file, encoding="utf-8")
    with raises(IncorrectFileException) as exception:
        check_type_hints([file])
    assert "file737ny73814781.py" in str(exception)


def test_filter_files() -> None:
    """Test filtering files by regex"""
    file_list = ["file1.py", "file2.txt", "excluded/dir/file3.py", "", "test_file4.py"]
    result = ["file1.py"]
    pattern = r"(excluded/|test_)"
    assert filter_files(file_list, pattern) == result


def test_filter_files_no_exclude() -> None:
    """Test default file filter"""
    file_list = ["file1.py", "file2.txt", "excluded/dir/file3.py", "", "test_file4.py"]
    result = ["file1.py", "excluded/dir/file3.py", "test_file4.py"]
    pattern = ""
    assert filter_files(file_list, pattern) == result


def test_filepath_in_log(caplog) -> None:
    """Test if the path to th file appears in the log"""
    with caplog.at_level(logging.INFO):
        check_type_hints([NO_RETURN])
    assert NO_RETURN in caplog.text


@pytest.mark.parametrize(
    "pattern,result",
    [
        ("^test", True),
        ("", False),
    ],
)
def test_exclude_parameters_by_regex(pattern, result):
    """Test if parameters are excluded"""
    assert (
        check_type_hints([MIXED_ARGS_WITH_RETURN], exclude_parameters=pattern) == result
    )


@pytest.mark.parametrize(
    "input_file,pattern,result",
    [
        (MIXED_ARGS, "", False),
        (NO_RETURN, "", False),
        (MIXED_ARGS_WITH_RETURN, "", False),
        (MIXED_ARGS, "dgag", False),
        (NO_RETURN, "adsg", False),
        (MIXED_ARGS_WITH_RETURN, "agddfa", False),
        (MIXED_ARGS, "^f", True),
        (NO_RETURN, "^f", True),
        (MIXED_ARGS_WITH_RETURN, "^f", True),
    ],
)
def test_exclude_by_name(input_file, pattern, result):
    """Test excluding functions and classes by name"""
    assert check_type_hints([input_file], exclude_by_name=pattern) == result


def testing_multiple_files(caplog):
    file_list = [MIXED_ARGS, NO_RETURN, MIXED_ARGS_WITH_RETURN]
    with caplog.at_level(logging.INFO):
        check_type_hints(file_list)
        assert all(filename in caplog.text for filename in file_list)


@pytest.mark.parametrize(
    "input_path,result",
    [
        (EMPTY_CLASS, True),
        (MIXED_ARGS_CLASS, False),
        (NO_RETURN_CLASS, False),
        (PROPERLY_ANNOTATED_CLASS, True),
        (STATIC_FUNCTION_CLASS, False),
    ],
)
def test_in_a_class(input_path, result) -> None:
    assert check_type_hints([input_path]) == result


def test_exclude_self() -> None:
    assert check_type_hints([STATIC_FUNCTION_CLASS], exclude_parameters="") == False
    assert check_type_hints([PROPERLY_ANNOTATED_CLASS], exclude_parameters="") == False
    assert check_type_hints([PROPERLY_ANNOTATED_CLASS]) == True
    assert check_type_hints([ANNOTATED_SELF_CLASS], exclude_parameters="") == True



import os
import subprocess
from typing import Set

RUN_ALL_OUTPUT = {
    "",
    "INFO:type_hint_checker:tests/cases/mixed_parameters.py: Missing type hint for "
    "parameter a "
    "(function f1), line 1",
    "INFO:type_hint_checker:tests/cases/mixed_parameters.py: Missing return type hint "
    "for function f1, line 1",
    "INFO:type_hint_checker:tests/cases/no_return.py: Missing return type hint "
    "for "
    "function f1, line 1",
    "INFO:type_hint_checker:tests/cases/mixed_parameters_with_return.py: Missing "
    "type hint for "
    "parameter test_aa (function f5), line 1",
    "INFO:type_hint_checker:tests/cases/comment_above.py: Missing type hint for "
    "parameter a (function f1), line 5",
    "INFO:type_hint_checker:tests/cases/comment_above.py: Missing type hint for "
    "parameter b (function f1), line 5",
    "INFO:type_hint_checker:tests/cases/comment_above.py: Missing return "
    "type hint for function f1, line 5",
    "INFO:type_hint_checker:tests/cases/comment_below.py: Missing type hint for "
    "parameter a (function f1), line 4",
    "INFO:type_hint_checker:tests/cases/comment_below.py: Missing type hint for "
    "parameter b (function f1), line 4",
    "INFO:type_hint_checker:tests/cases/comment_below.py: Missing return "
    "type hint for function f1, line 4",
    "INFO:type_hint_checker:tests/cases/different_comment.py: Missing type hint "
    "for parameter a (function f1), line 4",
    "INFO:type_hint_checker:tests/cases/different_comment.py: Missing type hint "
    "for parameter b (function f1), line 4",
    "INFO:type_hint_checker:tests/cases/different_comment.py: Missing return "
    "type hint for function f1, line 4",
    "INFO:type_hint_checker:tests/cases/mixed_parameters_class.py: Missing type hint "
    "for parameter a (function f1), line 4",
    "INFO:type_hint_checker:tests/cases/mixed_parameters_class.py: Missing return "
    "type hint for function f1, line 4",
    "INFO:type_hint_checker:tests/cases/no_return_class.py: Missing return "
    "type hint for function f1, line 4",
    "INFO:type_hint_checker:tests/cases/static_function_class.py: Missing "
    "type hint for parameter a (function f1), line 5",
}


def run_command(command: str) -> subprocess.CompletedProcess:
    """Common interface for executing command line programs"""
    return subprocess.run(
        command.split(), capture_output=True, universal_newlines=True, check=False
    )


def test_run_all() -> None:
    """Basic check"""
    process = run_command(
        "pre-commit run type_hint_checker --all-files -c tests/configs/strict.yaml"
    )
    output = process.stdout
    lines = prepare_output(output)
    assert lines == {
        *RUN_ALL_OUTPUT,
        "type_hint_checker........................................................"
        "Failed",
        "- hook id: type_hint_checker",
        "- exit code: 1",
    }
    assert process.returncode == 1


def test_run_all_not_strict() -> None:
    """Test if pre-commit doesn't fail when strict=False"""
    process = run_command(
        "pre-commit run type_hint_checker --all-files -c tests/configs/not-strict.yaml"
    )
    output = process.stdout
    lines = prepare_output(output)
    assert lines == {
        *RUN_ALL_OUTPUT,
        "type_hint_checker........................................................"
        "Passed",
        "- hook id: type_hint_checker",
    }
    assert process.returncode == 0


def prepare_output(output: str) -> Set[str]:
    """Prepares the output of the pylint program by splitting the lines, trimming and
    removing unnecessary output
    Parameters:
        output (str) - output of the command line program"""
    lines = output.split("\n")
    trimmed = [line.strip() for line in lines]
    result = []
    not_needed_phrases = [
        "- duration: ",
        "Once installed this environment will be reused.",
        "This may take a few minutes...",
        "Installing environment for ",
        "Initializing environment for",
        "The 'rev' field of ",
    ]
    for line in trimmed:
        if not any([phrase in line for phrase in not_needed_phrases]):
            result += [line]
    return set(result)



import argparse
import re
import sys
from typing import List
import logging

from type_hint_checker.checkers import FunctionChecker, ClassChecker
from type_hint_checker.file_parser import FileParser

logger = logging.getLogger("type_hint_checker")
logging.basicConfig()


def check_type_hints(
    file_list: List[str],
    exclude_parameters: str = "^self$",
    exclude_by_name: str = "",
    ignore_comment: str = "no-check",
) -> bool:
    """
    Iterates through the list of file paths, parses the files and checks if all
    functions and classes in the files have type hints.
    Parameters
    ----------
        file_list: List[str] - Filenames to be checked by
        exclude_parameters: str - regex specifying which parameters should not be
                            checked
        exclude_by_name: str - Regex specifying names of functions, methods and classes
                            that should not be checked
        ignore_comment : str - if this phrase appears in the comment, the item is
                                not checked for type hints presence
    Returns
    ----------
        True if all files have type hints.
    """
    result = True
    for filename in file_list:
        file = FileParser(
            filename,
            excluded_names=exclude_by_name,
            ignore_comment=ignore_comment,
        )
        function_checker = FunctionChecker(exclude_parameters=exclude_parameters)
        class_checker = ClassChecker(exclude_parameters=exclude_parameters)

        for function in file.functions:
            result = function_checker.check(function) and result
            function_checker.log_results(logger, filename=filename)
        for class_ in file.classes:
            result = class_checker.check(class_) and result
            class_checker.log_results(logger, filename=filename)
    return result


def parse_arguments() -> argparse.Namespace:
    """
    Parses command line arguments.
    """
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "filenames", help="Files to be checked by type_hint_checker.", nargs="+"
    )
    parser.add_argument(
        "--exit_zero",
        action="store_true",
        help="If this flag is checked, the program always exits with 0 (success) code.",
    )
    parser.add_argument(
        "--exclude_files",
        help="Regex specifying which files should not be checked",
        type=str,
        default="",
    )
    parser.add_argument(
        "--exclude_parameters",
        help="Regex specifying which parameters should not be checked",
        type=str,
        default="^self$",
    )
    parser.add_argument(
        "--exclude_by_name",
        help="Regex specifying names of functions, methods and classes that should not "
        "be checked",
        type=str,
        default="",
    )
    parser.add_argument(
        "--log-level",
        help="Controls how the amount of log messages",
        type=str,
        default="INFO",
        choices=["INFO", "DEBUG"],
    )
    parser.add_argument(
        "--ignore_comment",
        help="If this phrase appears in the comment, the item is excluded. Default : "
        "'no-check'",
        type=str,
        default="no-check",
    )

    args = parser.parse_args()
    return args


def filter_files(files: List[str], exclude_pattern: str) -> List[str]:
    """
    Filters the list of files passed by pre-commit hook to exclude files by a regex.
    Returns only filenames ending with .py
    Parameters
    ----------
        files (List[str]): Files to be checked
        exclude_pattern (str): Regex specifying which parameters should not be checked

    Returns
    -------
        List(str)
            list of files ending with .py and not excluded by the pattern
    """
    result = []
    for filename in files:
        if not exclude_pattern or not re.search(exclude_pattern, filename):
            if filename.endswith(".py"):
                result.append(filename)
    return result


def main() -> None:
    """Reads the command line arguments and runs the type_hint_checker"""
    args = parse_arguments()
    logger.setLevel(args.log_level)
    logger.debug(vars(args))
    files = filter_files(files=args.filenames, exclude_pattern=args.exclude_files)
    logger.debug("Files: %s", files)
    exit_code = 1 - check_type_hints(
        files,
        exclude_parameters=args.exclude_parameters,
        exclude_by_name=args.exclude_by_name,
        ignore_comment=args.ignore_comment,
    )
    if not args.exit_zero and exit_code:
        sys.exit(exit_code)


if __name__ == "__main__":
    main()



import ast
import re
from tokenize import tokenize, COMMENT
from typing import List

from type_hint_checker.exceptions import IncorrectFileException


class FileParser:  # pylint: disable=too-few-public-methods
    """
    File with its AST, functions, classes and exclusions by comments
    Parameters
    ----------
        filename : str - path to the file
        excluded_names : str - regex specifying which functions and classes should be
                                omitted
        ignore_comment : str - if this phrase appears in the comment, the item is
                                excluded
    """

    def __init__(
        self,
        filename: str,
        excluded_names: str = "",
        ignore_comment: str = "no-check",
    ) -> None:
        self.__ignore_comment = ignore_comment
        self.__excluded_names = excluded_names
        self.__filename = filename
        self.__body = self.__get_body()
        self.__excluded_lines = self.__get_excluded_lines()
        self.functions = self.__get_functions()
        self.classes = self.__get_classes()

    def __get_body(self) -> List[ast.AST]:
        """Parse the file into an Abstract Syntax Tree
        Returns
        -------
            List[ast.AST] - list of ast items from the file"""
        with open(self.__filename, "r", encoding="utf-8") as file:
            source = file.read()
        try:
            body = ast.parse(source).body
        except SyntaxError as exc:
            raise IncorrectFileException(
                f"File could not be parsed: {self.__filename}"
            ) from exc
        return body

    def __get_functions(self) -> List[ast.FunctionDef]:
        """Return functions defined in the file if not excluded
        Returns
        -------
            List[ast.FunctionDef] - list of ast.FunctionDefs from the file"""
        return [
            item
            for item in self.__body
            if isinstance(item, ast.FunctionDef)
            and not self.__is_excluded_by_comment(item)
            and not self.__is_excluded_by_name(item.name)
        ]

    def __get_classes(self) -> List[ast.ClassDef]:
        """Return classes defined in the file if not excluded
        Returns
        -------
            List[ast.ClassDef] - list of ast.ClassDefs from the file
        """
        return [
            item
            for item in self.__body
            if isinstance(item, ast.ClassDef)
            and not self.__is_excluded_by_comment(item)
            and not self.__is_excluded_by_name(item.name)
        ]

    def __get_excluded_lines(self) -> List[int]:
        """Return list of lines that are excluded from checking
        Returns
        ------
            List[int] - lines that are excluded

        """
        result = []
        with open(self.__filename, "rb") as file:
            tokenized = tokenize(file.readline)
            for item in tokenized:
                if item.exact_type == COMMENT:
                    if self.__ignore_comment in item.line:
                        result.append(item.start[0])
        return result

    def __is_excluded_by_comment(self, item: ast.AST) -> bool:
        """Return True if the item should be ommited
        Parameters
        _______
            item: ast item with fields .lineno and .end_lineno

        Returns
        -------
            bool - False if the object should not be checked"""
        for line in self.__excluded_lines:
            if item.lineno <= line <= item.end_lineno:
                return True
        return False

    def __is_excluded_by_name(self, name: str) -> bool:
        """
        Checks if the function or class should be checked
        Parameters
        ----------
        name (str): name of the function or class

        Returns
        -------
            bool - False if the object should not be checked
        """
        return self.__excluded_names and re.search(self.__excluded_names, name)



import ast
import re
from abc import ABC, abstractmethod
from logging import Logger
from typing import List, Optional, Union


class Checker(ABC):
    """Checks if an object is chas type hints.
    Parameters
    ----------
        exclude_parameters (str): regex specifying which parameters should not be
                                checked
        exclude_by_name: str - Regex specifying names of functions, methods and classes
                                that should not be checked
    """

    def __init__(
        self,
        exclude_parameters: str = "^self$",
        exclude_by_name: str = "",
    ) -> None:
        self._errors = []
        self._exclude_parameters = exclude_parameters
        self._exclude_by_name = exclude_by_name

    @abstractmethod
    def check(self, item: Union[ast.FunctionDef, ast.ClassDef]) -> bool:
        """
        Returns True if a given function/method has type hints.
        Parameters
        ----------
            item (Union[ast.FunctionDef, ast.ClassDef]): the object to be checked
        Returns
        -------
            Bool
        """

    def log_results(self, logger: Logger, filename: Optional[str] = None) -> None:
        """
        Displays a log message for each function or method with type hints.
        Parameters
        ----------
            logger (Logger): logger object that displays the message.
            filename (Optional[str]): If provided, the filename will be
                        prepended to the log message
        Returns
        -------
            None
        """
        prefix = ""
        if filename:
            prefix = f"{filename}: "
        for error in self._errors:
            logger.info(f"{prefix}{error}")

    def get_errors(self) -> List[str]:
        """
        Returns list of string describing the errors detected.
        """
        return self._errors


class FunctionChecker(Checker):
    """Checks if a function is has type hints.
    Parameters
    ----------
        exclude_parameters (str): regex specifying which parameters should not be
                                checked
        exclude_by_name: str - Regex specifying names of functions, methods and classes
                                that should not be checked
    """

    def __init__(
        self,
        exclude_parameters: str = "",
        exclude_by_name: str = "",
    ) -> None:
        super().__init__(
            exclude_parameters=exclude_parameters,
            exclude_by_name=exclude_by_name,
        )

    def check(self, item: ast.FunctionDef) -> bool:
        """
        Checks that the function has type hints (parameters and return type).
        Parameters
        ----------
            item (ast.FunctionDef): the function to be checked
        Returns
        -------
        bool
            True if type hints are present
        """
        self.__check_parameters(item)
        self.__check_return(item)
        return not bool(self._errors)

    def __check_parameters(self, function: ast.FunctionDef) -> None:
        """Check that the parameters of a function has type hints.
        Parameters
        ----------
            function (ast.FunctionDef): the function to be checked
        """
        parameters = function.args.args
        for parameter in parameters:
            if not parameter.annotation:
                if self.__check_if_param_should_be_checked(parameter.arg):
                    self._errors.append(
                        f"Missing type hint for parameter {parameter.arg} "
                        f"(function {function.name}), line {function.lineno}"
                    )

    def __check_if_param_should_be_checked(self, parameter: str) -> bool:
        """Returns True if the parameter should be checked.
        Parameters
        ----------
            parameter (str): - the parameters' name
        Returns
        ---------
            bool
        """
        return not self._exclude_parameters or not re.search(
            self._exclude_parameters, parameter
        )

    def __check_return(self, function: ast.FunctionDef) -> None:
        """Check that the function return type is provided.
        Parameters
        ----------
            function (ast.FunctionDef): string containing the source of the function to
                                        be checked
        """
        if not function.returns:
            self._errors.append(
                f"Missing return type hint for function {function.name}, "
                f"line {function.lineno}"
            )


class ClassChecker(Checker):
    """
    Checks if all methods in a given class has type hints.
    Parameters
    ----------
        exclude_parameters (str): regex specifying which parameters should not be
                                checked
        exclude_by_name: str - Regex specifying names of functions, methods and classes
                                that should not be checked
    """

    def __init__(
        self,
        exclude_parameters: List[str] = (),
        exclude_by_name: str = "",
    ) -> None:
        super().__init__(
            exclude_parameters=exclude_parameters,
            exclude_by_name=exclude_by_name,
        )

    def check(self, item: ast.ClassDef) -> bool:
        """
        Checks if all methods in a given class has type hints.
        Parameters
        ----------
            item (ast.FunctionDef): the class to be checked
        Returns
        -------
        bool
            True if all methods have type hints.
        """
        result = True
        for method in item.body:
            if isinstance(method, ast.FunctionDef):
                function_checker = FunctionChecker(
                    exclude_parameters=self._exclude_parameters,
                    exclude_by_name=self._exclude_by_name,
                )
                result = function_checker.check(method) and result
                self._errors += function_checker.get_errors()
        return result



import subprocess

import pytest

NO_RETURN = "tests/cases/no_return.py"
MIXED_ARGS = "tests/cases/mixed_parameters.py"
NO_ARGS = "tests/cases/no_parameters.py"
NOT_A_FUNCTION = "tests/cases/not_a_function.py"
MIXED_ARGS_WITH_RETURN = "tests/cases/mixed_parameters_with_return.py"
DIFFERENT_COMMENT = "tests/cases/different_comment.py"


def test_running_cli_version() -> None:
    subprocess.run(["type_hint_checker", NO_RETURN])


def test_exclude_files() -> None:
    process = subprocess.run(
        [
            "type_hint_checker",
            r"--exclude_files=no_.*\.py",
            NO_ARGS,
            NO_RETURN,
            NOT_A_FUNCTION,
        ],
        capture_output=True,
        universal_newlines=True,
    )
    assert process.returncode == 0


@pytest.mark.parametrize(
    "filename,result",
    [
        (MIXED_ARGS, 1),
        (NO_RETURN, 1),
        (NO_ARGS, 0),
        (NOT_A_FUNCTION, 0),
        (MIXED_ARGS_WITH_RETURN, 1),
    ],
)
def test_exit_code(filename: str, result: int) -> None:
    process = subprocess.run(
        ["type_hint_checker", filename],
        capture_output=True,
        universal_newlines=True,
    )
    assert result == process.returncode


@pytest.mark.parametrize(
    "filename",
    [
        MIXED_ARGS,
        NO_RETURN,
        MIXED_ARGS_WITH_RETURN,
    ],
)
def test_exit_code_with_exit_zero(filename: str) -> None:
    process = subprocess.run(
        ["type_hint_checker", filename, "--exit_zero"],
        capture_output=True,
        universal_newlines=True,
    )
    assert process.returncode == 0


def test_logging_filepath() -> None:
    process = subprocess.run(
        ["type_hint_checker", NO_RETURN],
        capture_output=True,
        universal_newlines=True,
    )
    assert NO_RETURN in process.stderr


@pytest.mark.parametrize(
    "pattern,result",
    [
        ("^test", 0),
        ("", 1),
    ],
)
def test_exclude_parameters(pattern: str, result: int) -> None:
    process = subprocess.run(
        [
            "type_hint_checker",
            MIXED_ARGS_WITH_RETURN,
            f"--exclude_parameters={pattern}",
        ],
        capture_output=True,
        universal_newlines=True,
    )
    assert result == process.returncode


@pytest.mark.parametrize(
    "input_,pattern,result",
    [
        (MIXED_ARGS, "", 1),
        (NO_RETURN, "", 1),
        (MIXED_ARGS_WITH_RETURN, "", 1),
        (MIXED_ARGS, "dgag", 1),
        (NO_RETURN, "adsg", 1),
        (MIXED_ARGS_WITH_RETURN, "agddfa", 1),
        (MIXED_ARGS, "^f", 0),
        (NO_RETURN, "^f", 0),
        (MIXED_ARGS_WITH_RETURN, "^f", 0),
    ],
)
def test_exclude_by_name(input_: str, pattern: str, result: int) -> None:
    """Test excluding functions and classes by name"""
    process = subprocess.run(
        [
            "type_hint_checker",
            input_,
            f"--exclude_by_name={pattern}",
        ],
        capture_output=True,
        universal_newlines=True,
    )
    assert result == process.returncode


def test_debug_level():
    process = subprocess.run(
        ["type_hint_checker", NO_RETURN, "--log-level=DEBUG"],
        capture_output=True,
        universal_newlines=True,
    )
    assert "DEBUG:type_hint_checker" in process.stderr
    assert "INFO:type_hint_checker" in process.stderr


def test_ignore_comment():
    process = subprocess.run(
        [
            "type_hint_checker",
            DIFFERENT_COMMENT,
            "--ignore_comment=custom",
        ],
        capture_output=True,
        universal_newlines=True,
    )
    assert process.returncode == 0
    process = subprocess.run(
        [
            "type_hint_checker",
            DIFFERENT_COMMENT,
        ],
        capture_output=True,
        universal_newlines=True,
    )
    assert process.returncode == 1
